<?php

namespace CRM\Factory;

use Doctrine\ORM\EntityManager;
use CRM\Traits\TQuery;

class CrmFaseDeNegociacaoFactory
{
    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getNegotiationPhaseFactory()
    {
        $sql = "SELECT
                    FASE.IDEXIBEFUNIL,
                    FASE.NRORDEM,
                    FASE.NRSEQFASENEGO,
                    FASE.NMSEQFASENEGO,
                    FASE.IDPERMITEFECHA,
                    FASE.IDEXIGEVROPORT,
                    FASE.IDATIVO,
                    FASE.NRORG,
                    FASE.NRORGINCLUSAO,
                    FASE.NRORGULTATU,
                    FASE.CDOPERINCLUSAO,
                    INC.NMOPERADOR AS NMOPERINCLUSAO,
                    FASE.CDOPERULTATU,
                    ALT.NMOPERADOR AS NMOPERULTATU,
                    FASE.DTINCLUSAO,
                    FASE.DTULTATU
                FROM
                    CRM_FASENEGOCI FASE
                    LEFT JOIN OPERADOR INC ON FASE.CDOPERINCLUSAO = INC.CDOPERADOR
                                            AND FASE.NRORG = INC.NRORG
                    LEFT JOIN OPERADOR ALT ON FASE.CDOPERULTATU = ALT.CDOPERADOR
                                            AND FASE.NRORG = ALT.NRORG
                ORDER BY
                    NRORDEM";

    return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function insertNegotiationPhaseFactory($params)
    {
        try {
            $sql = "INSERT INTO CRM_FASENEGOCI (
                        NRSEQFASENEGO,          IDEXIBEFUNIL,         NRORGINCLUSAO,
                        NMSEQFASENEGO,          NRORDEM,              NRORGULTATU,
                        IDPERMITEFECHA,         IDATIVO,              CDOPERINCLUSAO,
                        IDEXIGEVROPORT,         NRORG,                DTULTATU,
                        CDOPERULTATU,           DTINCLUSAO           
                    )
                    VALUES (
                        :NRSEQFASENEGO,          :IDEXIBEFUNIL,         :NRORGINCLUSAO,
                        :NMSEQFASENEGO,          :NRORDEM,              :NRORGULTATU,
                        :IDPERMITEFECHA,         :IDATIVO,              :CDOPERINCLUSAO,
                        :IDEXIGEVROPORT,         :NRORG,                SYSDATE,
                        :CDOPERULTATU,           SYSDATE          
            )";
            $this->connection->executeStatement($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function editNegotiationPhaseFactory($params)
    {
        try {
            $sql = "UPDATE CRM_FASENEGOCI
            SET
                IDEXIBEFUNIL = :IDEXIBEFUNIL,
                NMSEQFASENEGO = :NMSEQFASENEGO,
                IDPERMITEFECHA = :IDPERMITEFECHA,
                IDATIVO = :IDATIVO,
                CDOPERINCLUSAO = :CDOPERINCLUSAO,
                IDEXIGEVROPORT = :IDEXIGEVROPORT,
                NRORG = :NRORG,
                DTULTATU = SYSDATE,
                CDOPERULTATU = :CDOPERULTATU
            WHERE
                NRSEQFASENEGO = :NRSEQFASENEGO";
                
            $this->connection->executeStatement($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    public function deleteNegotiationPhase($params)
    {
        $sql = "DELETE
                FROM CRM_FASENEGOCI
                WHERE NRSEQFASENEGO IN (" . $params . ")";

        $this->connection->executeStatement($sql);
    } 
}
