/*****************************************************************************
* Sistemas Integrados TECNISA                                                *
*                                                                            *
* Compras e Suprimentos                                                      *
*                                                                            *
******************************************************************************
*                                                                            *
* Nome: CALC_DATA_ENTR_MES                                                   *
*                                                                            *
* Descrio:                                                                 *
* Calcula datas de entrega para um determinado mes baseando-se na            *
* Periodicidade de Entrega e nos Feriados de Compras.                        *
*                                                                            *
*                                                                            *
* Criao:                                                                   *
* 03/01/2000 - Yoris L. de Souza                                             *
*                                                                            *
* Manutenes:                                                               *
*                                                                            *
******************************************************************************

******************************************************************************
* PARAMETROS:                                                                *
*                                                                            *
* P_CDFILIAL     - Cdigo da Filial                                          *
* P_CDPERIENTR   - Cdigo da Periodicidade de Entrega                        *
* P_DTINICIAL    - Data do Inicio do Mes                                     *
* P_NRMESES      - Numero de meses a serem calculados                        *
******************************************************************************
* @D:\SQL\CALECOMP\8050_CMPSTP_CALC_DATA_ENTR_MES.SQL                        *
* @H:\SISTEMAS\DBA\SQL\ORACLE\8050_CMPSTP_CALC_DATA_ENTR_MES.SQL             *
******************************************************************************/

IF EXISTS(SELECT * FROM sysobjects O, sysusers U
          WHERE
            o.name = 'CALC_DATA_ENTR_MES'  and
            type = 'P'                  and
            u.uid = o.uid               and
            u.name = USER)
  DROP PROCEDURE CALC_DATA_ENTR_MES
  GO

CREATE PROCEDURE CALC_DATA_ENTR_MES
   @P_CDFILIAL     VARCHAR(4),
   @P_CDPERIENTR   VARCHAR(2),
   @P_DTINICIAL    DATETIME,
   @P_NRMESES      INT,
   @P_DTPROCESSA   DATETIME
AS
   DECLARE @V_CONTAMES   INT;
   DECLARE @V_IDTIPODIA  VARCHAR(1);
   DECLARE @V_IDDIANUTIL VARCHAR(1);
   DECLARE @V_DTINIAUX   DATETIME;
   DECLARE @V_DTENTREGA  DATETIME;
   DECLARE @V_ENTREGA    DATETIME;
   DECLARE @V_DTANTPROX  DATETIME;
   DECLARE @V_CONTA      NUMERIC(10);
   DECLARE @V_ANO        VARCHAR(4);
   DECLARE @V_DTRECORRENTE DATETIME;
   DECLARE @V_MAIORENTREGA DATETIME;
   DECLARE @V_DTANTPROXENT DATETIME;
BEGIN

  SELECT @V_IDTIPODIA  = IDTIPODIA,
         @V_IDDIANUTIL = IDDIANUTIL
  FROM PERIENTR
  WHERE
  CDPERIENTR = @P_CDPERIENTR;

  SET @V_DTINIAUX = @P_DTINICIAL;
  SET @V_ANO = SUBSTRING(CONVERT(VARCHAR, @P_DTINICIAL, 103), 7, 4);
  SET @V_DTRECORRENTE = CONVERT(DATETIME, '01/01/'+@V_ANO, 103);

  IF (@V_IDTIPODIA = 'R') BEGIN
     EXEC SEMANAL_RECORRENTE_DTENTR @P_CDFILIAL, @P_CDPERIENTR, @V_DTRECORRENTE, 12, @P_DTPROCESSA;
  END
  ELSE BEGIN
     SET @V_CONTAMES = 1;
     WHILE (@V_CONTAMES <= @P_NRMESES)
     BEGIN
       SET @V_CONTA = 0;
       SELECT @V_CONTA = COUNT(*)
       FROM CALECOMP
       WHERE
          CDFILIAL         = @P_CDFILIAL   AND
          CDPERIENTR       = @P_CDPERIENTR AND
          MONTH(DTENTREGA) = MONTH(@V_DTINIAUX) AND
          YEAR(DTENTREGA)  = YEAR(@V_DTINIAUX);

       IF (@V_CONTA > 0)
       BEGIN
          DELETE FROM CALECOMP
          WHERE
            CDFILIAL          = @P_CDFILIAL   AND
            CDPERIENTR        = @P_CDPERIENTR AND
            DTENTREGA        >= @P_DTPROCESSA AND
            MONTH(DTENTREGA)  = MONTH(@V_DTINIAUX) AND
            YEAR(DTENTREGA)   = YEAR(@V_DTINIAUX);
       END;

       IF (@V_IDTIPODIA = 'M')
       BEGIN
          EXEC MENSAL_DATA_ENTR_MES @P_CDFILIAL,@P_CDPERIENTR,@V_DTINIAUX,@V_IDDIANUTIL,@P_DTPROCESSA;
       END
       ELSE
       BEGIN
          IF (@V_IDTIPODIA = 'S') BEGIN
             EXEC SEMANAL_DATA_ENTR_MES @P_CDFILIAL,@P_CDPERIENTR,@V_DTINIAUX,@V_IDDIANUTIL,@P_DTPROCESSA;
          END;
       END;

       EXEC ULTIMO_DIA_MES @P_DTDATA = @V_DTINIAUX OUTPUT;

       SET @V_DTINIAUX = @V_DTINIAUX + 1;
       SET @V_CONTAMES = @V_CONTAMES + 1;
     END;

     SELECT @V_MAIORENTREGA = MAX(DTENTREGA)
       FROM CALECOMP
      WHERE DTENTREGA < @P_DTINICIAL
        AND CDFILIAL = @P_CDFILIAL
        AND CDPERIENTR = @P_CDPERIENTR;

     SET @V_ENTREGA = NULL;
     DECLARE C_CALECOMP CURSOR FOR
        SELECT
          DTENTREGA
        FROM
          CALECOMP
        WHERE CDFILIAL   = @P_CDFILIAL
          AND CDPERIENTR = @P_CDPERIENTR
          AND DTENTREGA >= @V_MAIORENTREGA
          AND DTENTREGA < @V_DTINIAUX
        ORDER BY DTENTREGA;
      OPEN C_CALECOMP;
      FETCH NEXT FROM C_CALECOMP INTO @V_DTENTREGA;
      WHILE (@@FETCH_STATUS = 0)
      BEGIN
        IF @V_ENTREGA IS NOT NULL BEGIN
           UPDATE
             CALECOMP
           SET
             DTANTPROXENT = @V_DTENTREGA - 1
           WHERE
             CDFILIAL   = @P_CDFILIAL   AND
             CDPERIENTR = @P_CDPERIENTR AND
             DTENTREGA  = @V_ENTREGA;
        END;
        SET @V_ENTREGA = @V_DTENTREGA;
        FETCH NEXT FROM C_CALECOMP INTO @V_DTENTREGA;
      END;
      CLOSE C_CALECOMP;
      DEALLOCATE C_CALECOMP;

      SELECT @V_DTANTPROXENT = CONVERT(DATETIME, MIN(DTENTREGA)-1)
        FROM CALECOMP
       WHERE CDFILIAL   = @P_CDFILIAL
         AND CDPERIENTR = @P_CDPERIENTR
         AND DTENTREGA  > @V_ENTREGA;

      IF @V_DTANTPROXENT IS NOT NULL BEGIN
         UPDATE
           CALECOMP
         SET
           DTANTPROXENT = @V_DTANTPROXENT
         WHERE
           CDFILIAL   = @P_CDFILIAL   AND
           CDPERIENTR = @P_CDPERIENTR AND
           DTENTREGA  = @V_ENTREGA;
      END;
  END;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'CALC_DATA_ENTR_MES','5.05.021';
GO
